// -*- c++ -*-
/***************************************************************************                          guicommandline.h  -  description
                             -------------------
    begin                : jeu avr 18 16:56:00 CET 2002
    copyright            : (C) 2002 by Romain Vinot
    email                : vinot@aist.enst.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _GUICOMMANDLINE_H_
#define _GUICOMMANDLINE_H_

/*
  CommandLine is the GUI interface for player interaction.
  This class takes all the input of the player and if it's a command execute it

  This class is a virtual one. It must be inherited from xcommandline or 
  qcommandline.
*/

#include <string>
#include <set>
using namespace std;


class QString;

#include "variables.h"

class Board;
class GameManager;
class GUICreature;
class GUIBoard;
class DataStoring;

struct SmtpInfo {
  QString smtp;
  QString yourEmail;
  QString opponentEmail;
  QString playermsg;
  QString port;
  bool useAuth;
  QString login;
  QString password;
};

class GUICommandLine
{
public:
  GUICommandLine(GUIBoard *board, GUICreature *guicrea);
  virtual ~GUICommandLine(void);
  
  virtual void endOfExecution(void)=0;

  string getPreviousCommand(void);

  virtual void commandNewGame(const char *command);
  virtual string commandView(const char *command);
  virtual string commandSelect(const char *command);
  virtual string commandMove(const char *command);
  virtual string commandEnter(const char *command);
  virtual string commandRevert(const char *command);
  virtual string commandTurn(const char *command);
  virtual string commandOpen(const char *command);
  virtual string commandClose(const char *command);
  virtual string commandAttack(const char *command);
  virtual string commandShoot(const char *command);
  virtual string commandFire(const char *command);
  virtual string commandStop(const char *command);
  virtual string commandUnjam(const char *command);
  virtual string commandLoadFlamer(const char *command);
  virtual string commandOverwatch(const char *command);
  virtual string commandTakeOrDrop(const char *command);
  virtual string commandEndTurn(const char *command);
  virtual string commandSave(const char *command);
  virtual string commandLoad(const char *command);
  virtual string commandMessage(const char *command);
  virtual string commandConnect(const char *command);
  virtual string commandRecord(const char *command);
  virtual string commandUndo(const char *command);
  virtual string commandNetwork(const char *command);

  virtual pair<Direction,bool> GetDirection(string txt, bool enabReuse=true)=0;
  virtual QString GetLevelToPlay()=0;
  virtual pair<int,pair<Direction,bool> > 
    GetCaseAndDirection(set<int> *cas, string txt, bool enableReuse=true)=0;
  virtual int GetCase(set<int> *cases, string txt)=0;
  virtual bool GetYesOrNo(string txt)=0;
  virtual gameType GetGameType(bool *heurTurn, int *cheat)=0;
  virtual QString GetFilename(bool PbemExt=false)=0;
  virtual void GetSmtpInfos(SmtpInfo *inf)=0;
  virtual QString GetIPAdress()=0;
  virtual void endOfGame(string msg)=0;
  virtual void CenterView(int caseId)=0;


  virtual string commandHelp(const char *command);
  FullDirection getDirectionType(char *d);

  virtual void printLevelInfo(QString &tit, QString &auth, QString &nar,
			      QString &obj, QString &forc, QString &cond)=0;
  virtual void printLevelInfo(void)=0;
  virtual void Write(string w, bool isCommand=false)=0;
  virtual void DrawInterface(void)=0;
  virtual void ClearListCreatures(void)=0;
  virtual void printPopupMessage(string msg)=0;
  virtual void ClearInfoPanel(void)=0;

  virtual void getCommand(string command);
  virtual void getReplayCommand(string command);
  virtual bool getMainCommands(string command); 
  virtual void getAllCommands(string command);
  virtual void getShootingCommand(string command);
  virtual void getFiringCommand(string command);

  void setState(interfaceState s);

  virtual void setObjectButtonsVisible(bool vis)=0;
protected:
  GUIBoard *guiboard;
  GUICreature *guicrea;
  
  interfaceState state;

  string previous;
};

#endif
