/***************************************************************************
                          network.cpp  -  description
                             -------------------
    begin                : lun aug 21 15:20:00 CET 2003
    copyright            : (C) 2003 by Romain Vinot
    email                : vinot@aist.enst.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef _WIN32
#pragma warning (disable : 4786)
#endif

#ifdef _WIN32
#include "zlib.h"
#else
#include <zlib.h>
#endif

#include <qsocket.h>
#include <qstring.h>
#include <qtextstream.h>

#include "network.h"

#include "guicommandline.h"
#include "gamemanager.h"

NetworkServer::NetworkServer(GUICommandLine *gc)
  : QServerSocket(9678,1), guicmd(gc)
{}

NetworkServer::~NetworkServer() {}

void NetworkServer::newConnection(int socket)
{
  guicmd->Write("Connection received. The game can begin.");
  man->setConnection(socket);
  guicmd->getCommand("network init");
}

NetworkInterface::NetworkInterface(GUICommandLine *gc, QString ip)
  : guicmd(gc), socket()
{
  socket = new QSocket();
  socket->connectToHost(ip,9678);
  stream = new QTextStream( socket );
  connect ( socket, SIGNAL( readyRead() ),
	    this, SLOT( readyRead() ) );
  connect ( socket, SIGNAL (hostFound() ),
	    this, SLOT( hostFound() ) );
  connect ( socket, SIGNAL (connected() ),
	    this, SLOT( connected() ) );
}

NetworkInterface::NetworkInterface(GUICommandLine *gc, int socketid)
  : guicmd(gc), socket()
{
  socket = new QSocket();
  socket->setSocket(socketid);
  stream = new QTextStream( socket );
  connect ( socket, SIGNAL( readyRead() ),
	    this, SLOT( readyRead() ) );
}

NetworkInterface::~NetworkInterface()
{
  delete stream;
  delete socket;
}

void NetworkInterface::hostFound()
{
    guicmd->Write("The host has been found.");
}

void NetworkInterface::connected()
{
    guicmd->Write("Connection done with the other player.\n"
  		"You will now receive the map.");
}

void NetworkInterface::readyRead()
{
  if ( !socket->canReadLine() )
    return;

  QString responseLine = socket->readLine();
  if (responseLine.startsWith("file begin ")) {
    QString response;
    while( socket->canReadLine() ) {
      responseLine = socket->readLine();
      if (responseLine.startsWith("file end "))
	break;
      response += responseLine;
    }
    guicmd->Write(response.latin1());
  }
  guicmd->Write(responseLine.latin1());
}

void NetworkInterface::sendFile(QString &filename, bool initgame)
{
  QString doc;
  gzFile f = gzopen(filename.latin1(),"rb");
  char car;
  while (gzread(f,&car,1)>0) {
    doc+=car;
  }
  gzclose(f);

  if (initgame)
    *stream << "file begin load\n" << doc << "file end init\n";
  else
    *stream << "file begin load\n" << doc << "file end init\n";
}
