/***************************************************************************
                          xcommandline.cpp  -  description
                             -------------------
    begin                : lun mar 04 14:59:00 CET 2002
    copyright            : (C) 2002 by Romain Vinot
    email                : vinot@aist.enst.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef _WIN32
#pragma warning (disable : 4786)
#endif

#include <cstdio>
#include <sstream>
#include <qstring.h>
using namespace std;

#include "xcommandline.h"

#include "gamemanager.h"
#include "xdatastoring.h"

XCommandLine::XCommandLine(XBoard *x, XCreature *c)
  : GUICommandLine(x, c)
{}

XCommandLine::~XCommandLine(void)
{}

void XCommandLine::endOfExecution(void)
{ exit(0); }

void XCommandLine::MainLoop(void)
{
  char command[1024];
  while (1) {
    cout << "> " << flush;
    cin.getline(command, 1023);

    getCommand(command);
  }
}

void XCommandLine::Write(string w, bool isCommand)
{
  if (!isCommand)
    cout << w << endl;
}

void XCommandLine::printLevelInfo(QString &title, QString &auth, QString &nar,
				  QString &obj, QString &forc, QString &cond)
{
  cout << "Level title : " << title.latin1() << endl << endl;
  cout << "Author : " << auth.latin1() << endl << endl;
  cout << "Description :\n" << nar.latin1() << endl << endl;
  cout << "Objectives : " << obj.latin1() << endl << endl;
  cout << "Forces : " << forc.latin1() << endl << endl;
  cout << "Win conditions : " << cond.latin1() << endl << endl;
}

void XCommandLine::printLevelInfo(void)
{
  QString tit, auth, nar, obj, forc, cond;
  man->getLevelInfo(tit,auth,nar,obj,forc,cond);
  cout << "Level title : " << tit.latin1() << endl << endl;
  cout << "Author : " << auth.latin1() << endl << endl;
  cout << "Description :\n" << nar.latin1() << endl << endl;
  cout << "Objectives : " << obj.latin1() << endl << endl;
  cout << "Forces : " << forc.latin1() << endl << endl;
  cout << "Win conditions : " << cond.latin1() << endl << endl;
}

void XCommandLine::endOfGame(string msg)
{ cout << msg << endl; }

void XCommandLine::printPopupMessage(string msg) 
{ cout << msg << endl; }

void XCommandLine::DrawInterface(void) {}
void XCommandLine::ClearListCreatures(void) {}
void XCommandLine::ClearInfoPanel(void) {}
void XCommandLine::setObjectButtonsVisible(bool vis) {}


pair<Direction,bool> XCommandLine::GetDirection(string txt, bool enableReuse)
{
  char answer[512];
  cout << txt << endl << "Direction must be : n,s,e or w.\n";
  if (man->isInReplayMode() && man->it_replay!=man->replays.end()) {
    const char *val = *(man->it_replay);
    char tmp1[100], tmp2[100], tmp3[100], tmp4[100];
    sscanf(val,"%s %s %s %s",tmp1,tmp2,tmp3,tmp4);
    if (strcmp(tmp1,"get")==0 && strcmp(tmp2,"direction")==0) {
      man->it_replay++;
      return pair<Direction,bool>(string2dir(string(tmp3)),
				  strcmp(tmp4,"reuse")==0); 
    }
  }
  while(1) {
    cout << "? " << flush;
  
    cin.getline(answer, 511);
    char c='d';
    sscanf(answer,"%c",&c);
    switch (c) {
    case 'n':
      return pair<Direction,bool>(NORTH,false);
    case 's':
      return pair<Direction,bool>(SOUTH,false);
    case 'e':
      return pair<Direction,bool>(EAST,false);
    case 'w':
      return pair<Direction,bool>(WEST,false);
    default:
      cout << "Position is not good. Please try again.\n";
    }
  }
}

QString XCommandLine::GetLevelToPlay(void)
{
  char path[1024];
  cout << "Please give the level filename to play :\n";
  cout << "? " << flush;
  
  char fname[128];
  cin.getline(fname, 127);
  sprintf(path,"%s%clevels%c%s",DATA_DIR,DIR_DELIMITER,DIR_DELIMITER, fname);
  return QString(path);
}

QString XCommandLine::GetFilename(bool PbemExt)
{
  if (PbemExt)
    cout << "Please give the filename to save pbem (.pbem extension) :\n";
  else
    cout << "Please give the filename to save game (.sav extension) :\n";
  cout << "? " << flush;
  
  char fname[128];
  cin.getline(fname, 127);
  QString fn = homeDir + DIR_DELIMITER + "savegame" + DIR_DELIMITER + fname;
  return fn;
}

QString XCommandLine::GetIPAdress()
{
  cout << "Please specify the IP adress of the other player:\n";
  cout << "? " << flush;
  
  char ip[128];
  cin.getline(ip, 127);
  return QString(ip);
}

pair<int,pair<Direction,bool> > 
XCommandLine::GetCaseAndDirection(set<int> *cases, string txt,
				  bool enableReuse)
{
  cout << txt << endl << _getStringCases(cases) << endl;
  if (man->isInReplayMode() && man->it_replay!=man->replays.end()) {
    const char *val = *(man->it_replay);
    char tmp1[100], tmp2[100], tmp3[100],tmp4[100];
    int nb;
    sscanf(val,"%s %s %d %s %s",tmp1,tmp2,&nb,tmp3,tmp4);
    if (strcmp(tmp1,"get")==0 && strcmp(tmp2,"casedir")==0) {
      man->it_replay++;
      return pair<int, pair<Direction,bool> > 
	(nb, pair<Direction,bool>
	 (string2dir(string(tmp3)),strcmp(tmp4,"reuse")==0));
    }
  }
  char answer[512];
  
  Direction dir=NODIRECTION;
  int caseId=-1;
  bool contin=true;;
  while(contin) {
    contin=false;
    cout << "? " << flush;
  
    cin.getline(answer, 511);
    int xpos=-2, ypos=-2;
    char d='d';
    sscanf(answer,"%d %d %c",&xpos, &ypos, &d);
    if (d=='n') dir=NORTH;
    else if(d=='s') dir=SOUTH;
    else if(d=='e') dir=EAST;
    else if(d=='w') dir=WEST;
    else {
      contin=true;
      cout << "Direction is not good, please give square and direction again.\n";
    }

    caseId = xpos+ypos*board->Xmax;
    if (cases->find(caseId) == cases->end()) {
      contin=true;
      cout << "Position is not good. Please give a correct square.\n";
    }    
  }
  return pair<int,pair<Direction,bool> >(caseId,
					 pair<Direction,bool>(dir,false));  
}

int XCommandLine::GetCase(set<int> *cases, string txt)
{
  cout << txt << endl << _getStringCases(cases) << endl;
  if (man->isInReplayMode() && man->it_replay!=man->replays.end()) {
    const char *val = *(man->it_replay);
    char tmp1[100], tmp2[100];
    int nb;
    sscanf(val,"%s %s %d",tmp1,tmp2,&nb);
    if (strcmp(tmp1,"get")==0 && strcmp(tmp2,"case")==0) {
      man->it_replay++;
      return nb; 
    }
  }
  char answer[512];
  
  int caseId=-1;
  bool contin=true;;
  while(contin) {
    contin=false;
    cout << "? " << flush;
  
    cin.getline(answer, 511);
    int xpos=-2, ypos=-2;
    sscanf(answer,"%d %d",&xpos, &ypos);
    caseId = xpos+ypos*board->Xmax;
    if (cases->find(caseId) == cases->end()) {
      contin=true;
      cout << "Position is not good. Please give a correct square.\n";
    }    
  }
  return caseId;  
}

bool XCommandLine::GetYesOrNo(string txt)
{
  cout << txt;
  if (man->isInReplayMode() && man->it_replay!=man->replays.end()) {
    const char *val = *(man->it_replay);
    char tmp1[100], tmp2[100];
    int nb;
    sscanf(val,"%s %s %d",tmp1,tmp2,&nb);
    if (strcmp(tmp1,"get")==0 && strcmp(tmp2,"yesorno")==0) {
      man->it_replay++;
      return nb; 
    }
  }
  char answer[1024];
  cout << "? " << flush;
  
  cin.getline(answer, 1023);
  char b;
  sscanf(answer, "%c", &b);
  if (b=='y') return true;
  else return false;
}

gameType XCommandLine::GetGameType(bool *heurisTurn, int *cheat)
{
  char answer[1024];
  char b;
  
  cout << "Do you want to use heuristic for turning successful defenser "
    "(y|n) ?\n";
  cout << "? " << flush;
  cin.getline(answer,1023);
  sscanf(answer,"%c",&b);
  *heurisTurn = (b=='y');

  bool ok=false;
  gameType gm=HOTSEAT;
  do {
    cout << "Which type of game do you want to play "
      "(solo,hotseat,pbem,network) ?\n";
    cin.getline(answer,1023);
    
    ok=true;
    if (strcmp(answer,"solo")==0)
      gm = SOLO;
    else if (strcmp(answer,"hotseat")==0)
      gm = HOTSEAT;
    else if (strcmp(answer,"pbem")==0)
      gm = PBEM;
    else if (strcmp(answer,"network")==0)
      gm = NETWORK;
    else
      ok=false;
  } while (!ok);

  if (gm==PBEM) {
    *cheat=1;
    cout << "Do you want to use cheat protection ?\n";
    cout << "? " << flush;
    cin.getline(answer, 511);
    sscanf(answer,"%d",cheat);
  }
  return gm;
}

void XCommandLine::CenterView(int caseId) {}

string XCommandLine::_getStringCases(set<int> *lst, int chunk)
{
  stringstream res;
  int idx=0;
  for (set<int>::iterator it=lst->begin(); it!=lst->end(); it++) {
    res << " [" << (*it)%board->Xmax << " " << (*it)/board->Xmax << "] ";
    idx++;
    if (chunk && idx%chunk == 0)
      res << "\n";
  }
  return res.str();
}

void XCommandLine::GetSmtpInfos(SmtpInfo *inf)
{
  char answer[1024];
  cout << "Give the internet adress of your SMTP server.\n? " << flush;
  cin.getline(answer,1023);
  inf->smtp = answer;

  cout << "Give your own email adress.\n? " << flush;
  cin.getline(answer,1023);
  inf->yourEmail = answer;

  cout << "Give your opponent's email adress.\n? " << flush;
  cin.getline(answer,1023);
  inf->opponentEmail = answer;

  inf->useAuth=false;
}
