/***************************************************************************
                          xcreature.cpp  -  description
                             -------------------
    begin                : ven mar 08 21:19:00 CET 2002
    copyright            : (C) 2002 by Romain Vinot
    email                : vinot@aist.enst.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "xcreature.h"

#include "board.h"

XCreature::XCreature(void)
{}

XCreature::~XCreature(void) 
{}

void XCreature::printInfo(Door *door, bool isSelected, int loc)
{
  cout << "Door : ";
  _printPosition(door);
  if (door->state == OPEN)
    cout << ", door open.\n";
  else
    cout << ", door closed.\n";
}

void XCreature::printInfo(Bulkhead *bulk, bool isSelected, int loc)
{
  cout << "Bulkhead : ";
  _printPosition(bulk);
  if (!bulk->stateCanBeOpened)
    cout << ", can never be opened";
  if (!bulk->stateCanBeClosed)
    cout << ", can never be closed";

  if (bulk->state == OPEN)
    cout << ", currently open.\n";
  else
    cout << ", currently closed.\n";
}

void XCreature::printInfo(Flame *fl, bool isSelected, int loc)
{
  cout << "Flame : ";
  _printPosition(fl);
  cout << ", nb of flames " << fl->nbFlames << ".\n";
}

void XCreature::printInfo(EntryZone *zone, bool isSelected, int loc)
{
  cout << "EntryZone [" << zone->getXPosition() << "," << zone->getYPosition();
  cout << "] with " << board->getObject(zone->getPosition())->size()-1 << " creatures :\n";
}

void XCreature::printInfo(BolterMarine *crea, bool isSelected, int loc)
{
  cout << crea->id << ", Marine : ";
  _printPosition(crea);
  cout << ", ";
  _printOrientation(crea);
  _printOverwatchMode(crea);

  if (isSelected) {
    int realActionPts = crea->actionPts - *(crea->cmdPts);
    if (realActionPts < 0)
      realActionPts = 0;
    
    cout << ", action pts " << realActionPts 
	 << ", command pts " << *(crea->cmdPts) << endl;  
  }
  else {
    cout << ", action pts " << crea->actionPts
	 << ", command pts " << *(crea->cmdPts) << endl;
  }
}

void XCreature::printInfo(Sergeant *crea, bool isSelected, int loc)
{
  cout << crea->id << ", Sergeant : ";
  _printPosition(crea);
  cout << ", ";
  _printOrientation(crea);
  _printOverwatchMode(crea);

  if (isSelected) {
    int realActionPts = crea->actionPts - *(crea->cmdPts);
    if (realActionPts < 0)
      realActionPts = 0;
    
    cout << ", action pts " << realActionPts 
	 << ", command pts " << *(crea->cmdPts) << endl;  
  }
  else {
    cout << ", action pts " << crea->actionPts
	 << ", command pts " << *(crea->cmdPts) << endl;
  }
}

void XCreature::printInfo(FlamerMarine *crea, bool isSelected, int loc)
{
  cout << crea->id << ", Flamer : ";
  _printPosition(crea);
  cout << ", ";
  _printOrientation(crea);
  cout << ", ammunition " << crea->ammunition;
  if (crea->charger)
    cout << ", charger " << crea->charger;

  if (isSelected) {
    int realActionPts = crea->actionPts - *(crea->cmdPts);
    if (realActionPts < 0)
      realActionPts = 0;
    
    cout << ", action pts " << realActionPts 
	 << ", command pts " << *(crea->cmdPts) << endl;  
  }
  else {
    cout << ", action pts " << crea->actionPts
	 << ", command pts " << *(crea->cmdPts) << endl;
  }
}

void XCreature::printInfo(Genestealer *crea, bool isSelected, int loc)
{
  cout << crea->id << ", Genestealer : ";
  _printPosition(crea);
  cout << ", ";
  _printOrientation(crea);
  cout << ", actionPts " << crea->actionPts << endl;
}

void XCreature::printInfo(Blip *crea, bool isSelected, int loc)
{
  cout << crea->id << ", Blip : ";
  _printPosition(crea);
  cout << ", actionPts " << crea->actionPts;
  if (crea->currentTurn==GENESTEALER_TURN)
    cout << ", nb genestealers : " << crea->nbGen;
  cout << endl;
}

void XCreature::printInfo(ModuleCAT *crea, bool isSelected, int loc)
{
  cout << crea->id << ", Module CAT : ";
  _printPosition(crea);
  cout << ", actionPts " << crea->actionPts;
  cout << " taken by a marine.";
  cout << endl;
}

void XCreature::printInfo(Exit *crea, bool isSelected, int loc)
{
  cout << "Exit square on ";
  _printPosition(crea);
  cout << ", already out : ";
  cout << endl;
}

void XCreature::printInfo(ExtensibleObject *crea, bool isSelected, int loc)
{
  cout << crea->desc.latin1() << " in ";
  _printPosition(crea);
  cout << endl;
}

// Do nothing, only useful for QT interface.
void XCreature::printDetailZoneInfo(EntryZone *ez)
{}

void XCreature::Message(string msg) { cout << msg << endl; }

void XCreature::_printPosition(Object *obj) {
  cout << "position " << obj->getXPosition() << " " << obj->getYPosition();  
}

void XCreature::_printOrientation(RealCreature *crea)
{
  char dir[10];
  switch (crea->orientation) {
  case NORTH:
    strcpy(dir,"NORTH");
    break;
  case SOUTH:
    strcpy(dir,"SOUTH");
    break;
  case EAST:
    strcpy(dir,"EAST");
    break;
  case WEST:
    strcpy(dir,"WEST");
    break;
  case NODIRECTION:
    strcpy(dir,"NODIR");
  }
  cout << "orientation " << dir;
}

void XCreature::_printOverwatchMode(BolterMarine *crea)
{
  if (crea->history==OVERWATCH)
    cout << ", in overwatch mode";
  if (crea->jammed)
    cout << ", bolter jammed";
}

void * XCreature::getPixmap(BolterMarine *crea, bool b) {return 0;}
void * XCreature::getPixmap(Sergeant *crea, bool b) {return 0;}
void * XCreature::getPixmap(FlamerMarine *crea, bool b) {return 0;}
void * XCreature::getPixmap(Genestealer *crea, bool b) {return 0;}
void * XCreature::getPixmap(Blip *crea, bool b) {return 0;}
void * XCreature::getPixmap(Door *crea, bool b) {return 0;}
void * XCreature::getPixmap(Bulkhead *crea, bool b) {return 0;}
void * XCreature::getPixmap(Flame *crea, bool b) {return 0;}
void * XCreature::getPixmap(EntryZone *crea, bool b) {return 0;}
void * XCreature::getPixmap(ModuleCAT *crea, bool b) {return 0;}
void * XCreature::getPixmap(Exit *crea, bool b) {return 0;}
void * XCreature::getPixmap(ExtensibleObject *crea, bool b) {return 0;}
